from rocolib.api.components.Component import Component

c = Component()
#N-WHEELED CAR (default 4-wheeed car)
#Probably need to put additional microcontroller in the middle for N > 8
n = 5 #(number of wheels/2)

#Add brain and wheels parameter (Default: Adafruit ESP32 2 Stacks)
c.addParameter("brains", "esp32stack", paramType="dimension")
c.addParameter("driveservo", "fs90r", paramType="dimension")

#The height will always match the height of the brains
c.addParameter("length", 80, paramType="length", minValue=80) #tested minValue
c.addParameter("width", 60, paramType="length", minValue=60) #tested minValue

#Connecting Each Car
for i in range(n):
    c.addSubcomponent("car%d" %i, "HalfCar")
    c.addConstraint(("car%d" %i, "driveservo"), "driveservo")
    c.addConstraint(("car%d" %i, "length"), ("length", "driveservo", "brains"), "(x[0] - (x[0]/2. - 2 * getDim(x[1], 'hornoffset')"
                                                                                 "- getDim(x[2], 'width')))")
    c.addConstraint(("car%d" %i, "height"), "brains", '(getDim(x, "height"))')
    c.addConstraint(("car%d" %i, "width"), "width")

for i in range(n - 1):
    c.addSubcomponent("between%d" %i, "SimpleRectBeam")
    c.addConstraint(("between%d" %i, "length"), ("width", "driveservo"), "x[0]- 2 * getDim(x[1], 'motorheight')")
    c.addConstraint(("between%d" %i, "depth"), "driveservo", "getDim(x, 'motorwidth')")
    c.addConstraint(("between%d" %i, "width"), "driveservo", "getDim(x, 'motorwidth')")
    c.addConnection(("between%d" %i, "topedge1"), ("car%d" %(i+1), "bright.topedge2"), angle=90)
    c.addConnection(("between%d" %i, "botedge1"), ("car%d"  %(i+1), "fleft.botedge2"), angle=90, tabWidth=10)

for i in range(n - 1):
    for j in range(3, 5):
        c.addConnection(("car%d" % i, "rect%d.r" % j), ("car%d" % (i + 1), "rect%d.l" % j))
    for j in range(1, 3):
        c.addConnection(("car%d" %i, "rect%d.l" %j), ("car%d" %(i+1), "rect%d.r" %j))

#Put shield for battery
#TODO: add a battery shield

# Putting the brains
c.addSubcomponent("holder", "ESP32Stack")
c.addConstraint(("holder", "length"), "width")

c.addSubcomponent("holdersplit", "SplitEdge")
c.addConstraint(("holdersplit", "toplength"), "brains", '(getDim(x, "height"),)')
c.addConstraint(("holdersplit", "botlength"),
                ("driveservo", "brains"),
                "(getDim(x[0], 'motorwidth'), getDim(x[1], 'height') - getDim(x[0], 'motorwidth'))")

c.addConnection(("holdersplit", "topedge0"), ("holder", "cover.topedge1"))
c.addConnection(("car0", "bright.topedge0"), ("holdersplit", "botedge0"), angle=-90)

#Putting between support
c.toLibrary("NWheeledCar")
